
# q-expansion of newform 1920.2.a.bb, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [-4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [1, 0], [1, 0], [1, 1], [2, 0], [1, 1], [3, -1], [1, 1], [-1, -1], [0, -2], [-3, -1], [-1, -1], [-2, 0], [2, -2], [-9, -1], [10, 0], [6, 0], [-2, 0], [2, -2], [-8, 0], [4, -2], [-9, 1], [-4, 0], [-10, 0], [10, 0], [14, 0], [1, -3], [-6, 2], [-4, 2], [11, -1], [-11, 1], [0, -2], [-7, 1], [3, 3], [12, 2], [5, 3], [-7, -3], [-8, 4], [-15, 1], [14, 0], [-4, -2], [8, -2], [-4, 0], [16, 2], [2, 0], [1, -5], [15, -1], [19, -1], [0, 0], [-10, 0], [5, 5], [2, -2], [-16, 2], [4, -2], [3, 7], [3, 3], [0, -2], [-3, -1], [-19, 1], [-8, 2], [22, 2], [-4, -6], [6, 2], [-22, -2], [-4, 6], [2, 0], [3, -1], [0, -6], [2, 2], [-6, 4], [1, -7], [4, 0], [-5, 3], [13, 1], [9, -3], [1, 1], [-6, 4], [-3, -3], [-20, 2], [2, 4], [-8, 2], [-2, 8], [0, 4], [-8, 2], [-15, 3], [-20, 4], [-14, 0], [-10, 4], [-32, -2], [33, 1], [6, -6], [20, 4], [7, -5], [-18, 4], [-17, 3], [19, -5], [14, 0], [10, 0], [-6, 6], [0, -2], [-18, 2], [18, -4], [2, -6], [0, 6], [23, -5], [10, 8], [-22, -6], [11, 7], [-8, -4], [2, -8], [-9, -1], [5, -3], [-11, 5], [-17, -1], [15, -7], [-28, -2], [-16, -4], [9, 1], [20, 6], [26, 0], [-2, 0], [-12, -2], [-20, 6], [-18, 2], [-9, -1], [-18, 0], [-18, -8], [14, 2], [-9, -5], [17, 5], [-29, -1], [-9, 7], [-15, 3], [-7, 5], [42, 0], [40, -2], [-2, 12], [30, 2], [20, -2], [-4, -2], [-13, -1], [0, 6], [33, -3], [40, -4], [4, -6], [44, 0], [17, 1], [35, -1], [21, 1], [-35, -3], [33, 5], [2, -4], [16, -4], [-27, 5], [-4, 8], [34, -6], [-35, -1], [-2, 4], [-14, 0], [-6, 12], [-2, -6], [41, 1], [39, 3], [24, 2], [35, -1], [35, 3], [11, 1], [15, -5], [-12, 2], [-16, -6], [26, 4], [0, -2], [-24, -8], [-40, 2], [-33, 1], [18, 4], [1, 9], [-8, -2], [-45, 3], [28, 2], [-35, -7], [-38, -4], [-13, -1], [-9, -5], [17, 9], [-10, 0], [15, -9], [12, 0], [42, -4], [-4, -8], [8, -6], [-30, 6], [11, -13], [20, -6], [-24, 0], [7, -5], [-34, 0], [25, -7], [3, -1], [3, -13], [-6, 12], [-5, 1], [-11, 5], [40, -6], [24, -2], [12, 2], [-15, 7], [42, 2], [-30, 4], [5, -3], [-12, 6], [24, 6], [-7, -11], [4, 8], [-2, -2], [-24, -10], [23, -1], [12, 2], [29, 5], [-36, -2], [22, 8], [-27, -9], [-14, -12], [21, -3], [-26, -6], [-24, -2], [-5, -1], [10, -10], [-51, 5], [20, 6], [-27, -3], [19, 7], [17, 3], [-12, 6], [10, -2], [7, -9], [4, -2], [16, 10], [-16, -10], [0, 16], [30, -2], [-53, -1], [-23, 5], [-6, -4], [-15, -7], [-26, 6], [23, 7], [18, -4], [-27, 9], [-51, -3], [-17, -5], [18, 0], [13, 5], [-16, 10], [-52, 2], [-54, 0], [0, -2], [-40, -4], [32, 6], [-36, -2], [-5, -1], [-18, 2], [6, 8], [-49, 7], [21, 5], [-21, 7], [-46, 4], [-44, -6], [38, -6], [-30, 4], [-4, -6], [-18, 2], [-48, -6], [47, -7], [4, -2], [-3, 1], [22, 6], [-42, -8], [24, 2], [-24, 6], [-39, 1], [3, -11], [13, -3], [36, 2], [-8, -4], [0, 12], [-42, -4], [-20, 6], [-53, 1], [-36, 10], [26, 12], [-44, -4], [-27, -11], [-4, -2], [11, 11], [24, -10], [23, -11], [-4, 10], [24, -2], [-14, -10], [16, 10], [18, -4], [5, 19], [2, -14], [33, 9], [26, -8], [-16, 4], [-62, -4], [-46, 10], [13, 17], [-27, 5], [36, -6], [-32, -2], [52, 0], [17, 9], [12, -6], [-36, -2], [-10, -2], [28, -2], [-24, -2], [13, 9], [24, 2], [-2, 0], [35, -9], [-37, -9], [10, -8], [31, 11], [-42, 10], [-41, 7], [-56, 2], [18, -12], [-30, -4], [-31, -1], [70, -2], [-35, 5], [4, 16], [-40, 6], [-39, -7], [14, -4], [-23, 5], [45, 1], [23, 3], [26, -4], [-21, -11], [64, -2], [-2, -4], [50, -4], [40, -12], [22, -10], [-54, -8], [21, -7], [2, -16], [2, -12], [11, -9], [-54, -4], [-51, 5], [88, 0], [32, -6], [-27, -3], [19, 11], [-21, -17], [16, -2], [19, 3], [-48, -2], [38, 2], [26, 8], [-18, 4], [-25, -1], [-38, -12], [36, 2], [21, -15], [-67, -3], [48, -10], [9, 15], [27, -1], [34, 8], [-6, 10], [-6, 8], [-34, -12], [-26, 12], [52, 2], [-27, -3], [33, 9], [21, -3], [31, 11], [-41, 15], [-15, 11], [93, 1], [46, 2], [-37, -5], [-36, -6], [6, 0], [10, -12], [30, 2], [-36, 4], [14, -4], [43, -11], [32, -6], [-15, -7], [-6, 4], [72, 6], [-31, 9], [29, -7], [37, 5], [16, 6], [-61, -3], [-5, 15], [-4, 2], [60, 0], [-24, -6], [-18, -4], [6, 16], [6, 18], [35, -13], [48, 2], [12, 10], [-52, 4], [-7, -7], [-31, 9], [19, 19], [-4, 10], [19, 7], [-33, -1], [-6, 12], [32, -6], [56, 6], [-4, -8], [-2, -20], [13, -11], [-64, -4]]
